use std::rc::Rc;

use derivative::Derivative;

use yew::prelude::*;
use yew::virtual_dom::Key;

/// List render callback arguments.
pub struct ListRenderArgs<'a, T> {
    pub(crate) record: &'a T,
    pub(crate) record_key: &'a Key,
    // Row index.
    pub(crate) row_index: usize,
    // Select flag is set when the row is selected.
    pub(crate) selected: bool,
}

impl<'a, T> ListRenderArgs<'a, T> {
    /// Return the data node.
    pub fn record(&self) -> &T {
        self.record
    }

    pub fn key(&self) -> &Key {
        self.record_key
    }

    /// Returns the row index.
    pub fn row_index(&self) -> usize {
        self.row_index
    }

    /// Returns if the cell is selected.
    pub fn is_selected(&self) -> bool {
        self.selected
    }
}

/// List render callback.
#[derive(Derivative)]
#[derivative(Clone(bound = ""), PartialEq(bound = ""))]
pub struct ListRenderCallback<T>(
    #[derivative(PartialEq(compare_with = "Rc::ptr_eq"))] Rc<dyn Fn(&ListRenderArgs<T>) -> Html>,
);

impl<T> ListRenderCallback<T> {
    /// Creates a new instance.
    pub fn new(renderer: impl 'static + Fn(&ListRenderArgs<T>) -> Html) -> Self {
        Self(Rc::new(renderer))
    }
    /// Apply the render function
    pub fn apply(&self, args: &ListRenderArgs<T>) -> Html {
        (self.0)(args)
    }
}

impl<T, F: 'static + Fn(&ListRenderArgs<T>) -> Html> From<F> for ListRenderCallback<T> {
    fn from(f: F) -> Self {
        ListRenderCallback::new(f)
    }
}
